unit chartoptu;

// Copyright  2001 by Ziff Davis Media, Inc.
// Written by Neil J. Rubenking

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls,
  Forms, Dialogs, teCanvas, TeeProcs, Series, TeEngine, Chart,
  jpeg, ExtDlgs, ComCtrls, StdCtrls, Buttons, ExtCtrls, dpShare, ToolWin,
  ImgList, Menus;

type
  TOptForm = class(TForm)
    pcOpt                : TPageControl;
      tsBack             : TTabSheet;
        rbSolid          : TRadioButton;
        Bevel1           : TBevel;
        rbGrad           : TRadioButton;
        Label1           : TLabel;
        cbGrad           : TComboBox;
        rbImage          : TRadioButton;
        btnBrowse        : TButton;
        cbImage          : TComboBox;
      tsSlice            : TTabSheet;
        gbColors         : TGroupBox;
          lbSlices       : TListBox;
          btnAdd         : TButton;
          btnDel         : TButton;
          btnUp          : TButton;
          btnDown        : TButton;
          btnChange      : TButton;
          btnDefault     : TButton;
        ebNum            : TEdit;
        udNum            : TUpDown;
      tsPie              : TTabSheet;
        btnFont          : TButton;
        cb3D             : TCheckBox;
        cbCircle         : TCheckBox;
        cbLegend         : TCheckBox;
        cbLabels         : TCheckBox;
        tbAngle          : TTrackBar;
        lbAngle          : TLabel;
        tbHeight         : TTrackBar;
        lbHeight         : TLabel;
    btnOK                : TButton;
    btnCancel            : TButton;
    cd1                  : TColorDialog;
    opd1                 : TOpenPictureDialog;
    fd1                  : TFontDialog;
    TabSheet1: TTabSheet;
    ilCols: TImageList;
    tbarSolid: TToolBar;
    tbSolid: TToolButton;
    btnSolid: TBitBtn;
    tbarGrad: TToolBar;
    tbStart: TToolButton;
    tbEnd: TToolButton;
    btnStart: TBitBtn;
    btnEnd: TBitBtn;
    tbSpace: TToolButton;
    Bevel2: TBevel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure tsBackMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure rbSolidClick(Sender: TObject);
    procedure btnColorAllClick(Sender: TObject);
    procedure rbGradClick(Sender: TObject);
    procedure cbGradChange(Sender: TObject);
    procedure rbImageClick(Sender: TObject);
    procedure btnBrowseClick(Sender: TObject);
    procedure cbImageChange(Sender: TObject);
    procedure lbSlicesClick(Sender: TObject);
    procedure lbSlicesDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure btnAddClick(Sender: TObject);
    procedure btnDelClick(Sender: TObject);
    procedure btnUpDownClick(Sender: TObject);
    procedure btnChangeClick(Sender: TObject);
    procedure btnDefaultClick(Sender: TObject);
    procedure udNumClick(Sender: TObject; Button: TUDBtnType);
    procedure btnFontClick(Sender: TObject);
    procedure cbCircleClick(Sender: TObject);
    procedure cb3DClick(Sender: TObject);
    procedure cbLegendClick(Sender: TObject);
    procedure cbLabelsClick(Sender: TObject);
    procedure tbAngleChange(Sender: TObject);
    procedure tbHeightChange(Sender: TObject);
    procedure fd1Apply(Sender: TObject; Wnd: HWND);
    procedure cd1Show(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure tbColorAllClick(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    { Private declarations }
    C          : TChart;
    fImgFile   : String;
    fHoldList  : TStringList;
    wasFont    : TFont;
    wasType    : Integer;
    wasSolid   : TColor;
    wasStart   : TColor;
    wasEnd     : TColor;
    wasGstyle  : Integer;
    wasIstyle  : Integer;
    wasImgFile : String;
    wasSlice   : TStringList;
    wasCirc    : Boolean;
    was3D      : Boolean;
    wasLegend  : Boolean;
    wasLabels  : Boolean;
    wasElev    : Integer;
    wasHigh    : Integer;
    wasSlices  : Integer;
    procedure SetAllEnabled;
    procedure ReSlice;
    procedure ColorImage(Idx : Integer; Col : TColor);
  public
    { Public declarations }
    procedure SetData(vC : TChart; pieXtra : TChartExtra;
      const vTitle  : String);
    procedure GetData(VAR pieXtra : TChartExtra);
  end;

var
  OptForm: TOptForm;

implementation

uses mainu, inifiles;

{$R *.dfm}

procedure TOptForm.FormCreate(Sender: TObject);

  procedure SetUpImages;
  VAR TB : TBitmap;
  begin
    TB := TBitmap.Create;
    try
      TB.Width := ilCols.Width;
      TB.Height := ilCols.Height;
      ilCols.Clear;
      ilCols.AddMasked(TB, clFuchsia);
      ilCols.AddMasked(TB, clFuchsia);
      ilCols.AddMasked(TB, clFuchsia);
    finally
      TB.Free;
    end;
  end;

  procedure FixButtonsFor95;
  // In Windows 95, the tbsDropDown style button can look weird,
  // with a huge triangle glyph. We just remove that style.
  begin
    IF Win32Platform <> VER_PLATFORM_WIN32_WINDOWS THEN Exit;
    IF Win32MajorVersion > 4 THEN Exit;
    IF Win32MinorVersion >= 10 THEN Exit;
    tbSolid.Style := tbsButton;
    tbStart.Style := tbsButton;
    tbEnd.Style := tbsButton;
  end;

begin
  FixButtonsFor95;
  WasSlice := TStringList.Create;
  WasFont  := TFont.Create;
  SetUpImages;
end;

procedure TOptForm.FormDestroy(Sender: TObject);
begin
  WITH TIniFile.Create(ininame) DO
  try
    WriteInteger(Self.Name + ' Settings', 'Page',
      pcOpt.ActivePageIndex);
  finally
    Free;
  end;
end;

procedure TOptForm.tsBackMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
// This method exists to give hints to disabled controls
VAR
  P : TPoint;
  N : Integer;
begin
  P := Point(X, Y);
  WITH Sender AS TWinControl DO
    begin
      Hint := '';
      FOR N := 0 TO ControlCount-1 DO
        IF Controls[N] IS TWinControl THEN
          IF PtInRect(TWinControl(Controls[N]).BoundsRect, P) THEN
            Hint := TWinControl(Controls[N]).Hint;
      IF Hint = '' THEN
        Application.CancelHint;
    end;
end;

procedure TOptForm.rbSolidClick(Sender: TObject);
// If they select solid, hide the gradient and destroy the image
begin
  C.Gradient.Visible        := False;
  C.BackImage.Bitmap.Handle := 0;
  SetAllEnabled;
end;

procedure TOptForm.btnColorAllClick(Sender: TObject);
// This routine is used by all three of the buttons that select
// color. After a successful new color choice, the button's glyph
// is redrawn.
begin
  CASE (Sender AS TBitBtn).Tag OF
    1 : cd1.Color := C.Color;
    2 : cd1.Color := C.Gradient.EndColor;
    3 : cd1.Color := C.Gradient.StartColor;
  END;
  IF cd1.Execute THEN
    begin
      CASE (Sender AS TBitBtn).Tag OF
        1 : C.Color               := cd1.Color;
        2 : C.Gradient.EndColor   := cd1.Color;
        3 : C.Gradient.StartColor := cd1.Color;
      END;
      WITH Sender AS TBitBtn DO
        begin
          Glyph.Canvas.Brush.Color := cd1.Color;
          Glyph.Canvas.Pen.Color   := clBlack;
          Glyph.Canvas.Rectangle(1, 1, 23, 14);
        end;
    end;
end;


procedure TOptForm.tbColorAllClick(Sender: TObject);
// This routine is used by all three of the buttons that select
// color. After a successful new color choice, the button's glyph
// is redrawn.
begin
  CASE (Sender AS TToolButton).Tag OF
    1 : cd1.Color := C.Color;
    2 : cd1.Color := C.Gradient.EndColor;
    3 : cd1.Color := C.Gradient.StartColor;
  END;
  IF cd1.Execute THEN
    begin
      CASE (Sender AS TToolButton).Tag OF
        1 : C.Color               := cd1.Color;
        2 : C.Gradient.EndColor   := cd1.Color;
        3 : C.Gradient.StartColor := cd1.Color;
      END;
      ColorImage((Sender AS TToolButton).ImageIndex, cd1.Color);
    end;
end;

procedure TOptForm.rbGradClick(Sender: TObject);
// If they select gradient, make the gradient visible and destroy
// the background image
begin
  C.Gradient.Visible        := True;
  C.BackImage.Bitmap.Handle := 0;
  SetAllEnabled;
end;

procedure TOptForm.cbGradChange(Sender: TObject);
// Translate our three simplified choices to TGradientDirection
const A : ARRAY[0..2] OF TGradientDirection = (gdBottomTop,
  gdRightLeft, gdFromCenter);
begin
  C.Gradient.Direction := A[(Sender AS TComboBox).ItemIndex];
end;

procedure TOptForm.rbImageClick(Sender: TObject);
// If they choose image, hide the gradient and load the image file
begin
  C.Gradient.Visible := False;
  IF FileExists(fImgFile) THEN
    C.BackImage.LoadFromFile(fImgFile);
  SetAllEnabled;
end;

procedure TOptForm.btnBrowseClick(Sender: TObject);
// Browse for an image file
begin
  opd1.FileName := fImgFile;
  IF opd1.Execute THEN
    begin
      fImgFile := opd1.FileName;
      C.BackImage.LoadFromFile(fImgFile);
    end;
end;

procedure TOptForm.cbImageChange(Sender: TObject);
begin
  WITH Sender AS TComboBox DO
    C.BackImageMode := TTeeBackImageMode(ItemIndex);
end;

procedure TOptForm.lbSlicesClick(Sender: TObject);
begin
  WITH Sender AS TListBox DO
    begin
      btnDel.Enabled    := Items.Count > 0;
      btnChange.Enabled := btnDel.Enabled;
      btnUp.Enabled     := btnDel.Enabled AND (ItemIndex > 0);
      btnDown.Enabled   := btnDel.Enabled AND (ItemIndex < Count-1);
    end;
end;

procedure TOptForm.lbSlicesDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
VAR R1, R2 : TRect;
begin
  R1       := Rect;
  R1.Right := R1.Left + 30;
  R2       := Rect;
  R2.Left  := R2.Left + 32;
  InflateRect(R2, -2, -2);
  WITH Control AS TListBox DO
    begin
      Canvas.FillRect(Rect);
      SetTextAlign(Canvas.Handle, TA_RIGHT);
      Canvas.TextRect(R1, R1.Right-2, R1.Top+2,
        IntToStr(Succ(Index)));
      Canvas.Brush.Color := TColor(Items.Objects[Index]);
      Canvas.FillRect(R2);
    end;
end;

procedure TOptForm.btnAddClick(Sender: TObject);
begin
  IF cd1.Execute THEN
    begin
      lbSlices.Items.AddObject(IntToStr(Succ(lbSlices.Items.Count)),
        Pointer(cd1.Color));
      lbSlices.ItemIndex := lbSlices.Items.Count-1;
      ReSlice;
    end;
end;

procedure TOptForm.btnDelClick(Sender: TObject);
VAR Idx : Integer;
begin
  IF lbSlices.ItemIndex < 0 THEN Exit;
  IF lbSlices.Count     < 2 THEN Exit;
  Idx := lbSlices.ItemIndex;
  lbSlices.Items.Delete(lbSlices.ItemIndex);
  IF Idx >= lbSlices.Count THEN Dec(Idx);
  lbSlices.ItemIndex := Idx;
  ReSlice;
end;

procedure TOptForm.btnUpDownClick(Sender: TObject);
VAR NewIndex : Integer;
begin
  WITH lbSlices DO
    IF (Sender AS TBUtton).Tag = 0 THEN
      begin
        IF ItemIndex > 0 THEN
          begin
            NewIndex := ItemIndex - 1;
            Items.Move(itemIndex, NewIndex);
            ItemIndex := NewIndex;
          end;
      end
    ELSE
      begin
        IF (ItemIndex >= 0) AND (ItemIndex < Items.Count-1) THEN
           begin
             NewIndex := ItemIndex + 1;
             Items.Move(ItemIndex, NewIndex);
             ItemIndex := NewIndex;
           end;
      end;
  ReSlice;
  btnUp.Enabled   := lbSlices.ItemIndex > 0;
  btnDown.Enabled := lbSlices.ItemIndex < lbSlices.Count-1;
end;

procedure TOptForm.btnChangeClick(Sender: TObject);
begin
  IF lbSlices.ItemIndex < 0 THEN Exit;
  cd1.Color := TColor(lbSlices.Items.Objects[lbSlices.ItemIndex]);
  IF cd1.Execute THEN
    begin
      lbSlices.Items.Objects[lbSlices.ItemIndex] :=
        Pointer(cd1.Color);
      ReSlice;
    end;
end;

procedure TOptForm.btnDefaultClick(Sender: TObject);
VAR N : Integer;
begin
  lbSlices.Clear;
  FOR N := 0 TO MaxDefaultColors-1 DO
    lbSlices.Items.AddObject(IntToStr(Succ(lbSlices.Items.Count)),
      Pointer(GetDefaultColor(N)));
  ReSlice;
end;

procedure TOptForm.udNumClick(Sender: TObject; Button: TUDBtnType);
begin
  MainForm.ListToSeries(fHoldList, lbSlices.Items,
    TPieSeries(C.Series[0]), udNum.Position);
end;

procedure TOptForm.btnFontClick(Sender: TObject);
begin
  fd1.Font := C.Title.Font;
  IF fd1.Execute THEN
    begin
      C.Title.Font := fd1.Font;
      C.Foot.Font := C.Title.Font;
    end;
end;

procedure TOptForm.cbCircleClick(Sender: TObject);
begin
  TPieSeries(C.Series[0]).Circled := (Sender AS TCheckBox).Checked;
end;

procedure TOptForm.cb3DClick(Sender: TObject);
begin
  C.View3D := (Sender AS TCheckBox).Checked;
end;

procedure TOptForm.cbLegendClick(Sender: TObject);
begin
  C.Legend.Visible := (Sender AS TCheckBox).Checked;
end;

procedure TOptForm.cbLabelsClick(Sender: TObject);
begin
  TPieSeries(C.Series[0]).Marks.Visible :=
    (Sender AS TCheckBox).Checked;
end;

procedure TOptForm.tbAngleChange(Sender: TObject);
begin
  C.View3DOptions.Elevation := (Sender AS TTrackBar).Position;
  lbAngle.Caption := IntToStr(C.View3DOptions.Elevation);
end;

procedure TOptForm.tbHeightChange(Sender: TObject);
begin
  C.Chart3DPercent := (Sender AS TTrackBar).Position;
  lbHeight.Caption := IntToStr(C.Chart3DPercent);
end;

procedure TOptForm.fd1Apply(Sender: TObject; Wnd: HWND);
begin
  C.Title.Font := (Sender AS TFontDialog).Font;
  C.Foot.Font := C.Title.Font;
end;

procedure TOptForm.cd1Show(Sender: TObject);
// When the color dialog appears, put it on the same side of the
// main program as the customize dialog
VAR
  dlg, work : TRect;
  L : Integer;
begin
  GetWindowRect((Sender AS TCommonDialog).Handle, dlg);
  SystemParametersInfo(SPI_GETWORKAREA, 0, @Work, 0);
  IF Left < TForm(Owner).Left THEN
    L := Work.Left
  ELSE
    L := Work.Right - (dlg.right-dlg.Left);
  SetWindowPos((Sender AS TCommonDialog).Handle, 0, L,
    Top + GetSystemMetrics(SM_CYCAPTION), 0, 0,
    SWP_NOSIZE OR SWP_NOZORDER);
end;

procedure TOptForm.btnCancelClick(Sender: TObject);
// Put everything back the way it was
begin
  lbSlices.Items.Assign(WasSlice);
  udNum.Position := wasSlices;
  MainForm.ListToSeries(fHoldList, lbSlices.Items,
    TPieSeries(C.Series[0]), udNum.Position);
  ReSlice;
  CASE WasType OF
    0 : begin
      C.BackImage.Bitmap.Handle := 0;
      C.Gradient.Visible        := False;
    end;
    1 : begin
      C.BackImage.Bitmap.Handle := 0;
      C.Gradient.Visible        := True;
    end;
    2 : begin
      C.BackImage.LoadFromFile(wasImgFile);
      C.Gradient.Visible := False;
    end;
  END;
  C.Color               := wasSolid;
  C.Gradient.StartColor := wasStart;
  C.Gradient.EndColor   := wasEnd;
  C.Gradient.Direction  := TGradientDirection(wasGstyle);
  C.BackImageMode       := TTeeBackImageMode(wasIstyle);
  C.Title.Font.Assign(WasFont);
  C.Foot.Font := C.Title.Font;
  TPieSeries(C.Series[0]).Circled       := WasCirc;
  C.View3D                              := Was3D;
  TPieSeries(C.Series[0]).Marks.Visible := wasLabels;
  C.Legend.Visible                      := wasLegend;
  C.View3DOptions.Elevation             := WasElev;
  C.Chart3DPercent                      := wasHigh;
  fImgFile                              := wasImgFile;
  ModalResult                           := mrCancel;
end;

procedure TOptForm.SetAllEnabled;
begin
  btnBrowse.Enabled := rbImage.Checked;
  cbImage.Enabled   := rbImage.Checked;
  btnStart.Enabled  := rbGrad.Checked;
  btnEnd.Enabled    := rbGrad.Checked;
  tbStart.Enabled  := rbGrad.Checked;
  tbEnd.Enabled    := rbGrad.Checked;
  cbGrad.Enabled    := rbGrad.Checked;
  btnSolid.Enabled  := rbSolid.Checked;
  tbSolid.Enabled  := rbSolid.Checked;
end;

procedure TOptForm.ReSlice;
VAR N : Integer;
begin
  // Apply the changes to the pie
  FOR N := 0 TO C.Series[0].Count-1 DO
    TPieSeries(C.Series[0]).ValueColor[N] :=
      TColor(lbSlices.Items.Objects[N MOD lbSlices.Count]);
  btnAdd.Enabled := lbSlices.Items.Count < udNum.Max;
end;

procedure TOptForm.SetData(vC: TChart; pieXtra : TChartExtra;
  const vTitle : String);
// Set up the customize form to match the pie's current settings.
// Save all information needed to restore those settings.

  procedure InitButton(B : TBitBtn; Col : TColor);
  begin
    B.Glyph.Canvas.Brush.Color := Col;
    B.Glyph.Canvas.Pen.Color   := clBlack;
    B.Glyph.Canvas.Rectangle(1, 1, 23, 14);
  end;

begin
  fHoldList    := pieXtra.dList;
  Self.Caption := vTitle;
  C            := vC;
  fImgFile     := '';
  wasType      := pieXtra.dBak;
  fImgFile     := pieXtra.dImage;
  IF (wasType = 2) AND ((fImgFile = '') OR
    (NOT FileExists(fImgFile))) THEN
    wasType := 1;
  CASE wasType OF
    0 : rbSolid.Checked := True;
    1 : rbGrad.Checked  := True;
    2 : rbImage.Checked := True;
  END;
  lbSlices.Items.Assign(pieXtra.dCols);
  wasSlice.Assign(pieXtra.dCols);
  wasFont.Assign(C.Title.Font);
  wasSlices  := pieXtra.dSlices;
  wasSolid   := C.Color;
  wasStart   := C.Gradient.StartColor;
  wasEnd     := C.Gradient.EndColor;
  wasGstyle  := Integer(C.Gradient.Direction);
  wasIstyle  := Integer(C.BackImageMode);
  wasImgFile := fImgFile;
  wasCirc    := TPieSeries(C.Series[0]).Circled;
  was3D      := C.View3D;
  wasLegend  := C.Legend.Visible;
  wasLabels  := TPieSeries(C.Series[0]).Marks.Visible;
  wasElev    := C.View3DOptions.Elevation;
  wasHigh    := C.Chart3DPercent;
  lbAngle.Caption   := IntToStr(C.View3DOptions.Elevation);
  lbHeight.Caption  := IntToStr(C.Chart3DPercent);
  cbCircle.Checked  := WasCirc;
  cb3D.Checked      := Was3D;
  cbLegend.Checked  := wasLegend;
  cbLabels.Checked  := wasLabels;
  tbAngle.Position  := WasElev;
  tbHeight.Position := WasHigh;
  udNum.Position    := pieXtra.dSlices;
  InitButton(btnSolid, WasSolid);
  InitButton(btnStart, WasEnd);
  InitButton(btnEnd, WasStart);
  ColorImage(tbSolid.ImageIndex, wasSolid);
  ColorImage(tbStart.ImageIndex, wasEnd);
  ColorImage(tbEnd.ImageIndex, wasStart);
  CASE WasGStyle OF
    0, 1 : cbGrad.ItemIndex := 0;
    2, 3 : cbGrad.ItemIndex := 1;
    ELSE cbGrad.ItemIndex   := wasGStyle - 2;
  END;
  cbImage.ItemIndex := WasIStyle;
  SetAllEnabled;
  btnAdd.Enabled    := lbSlices.Items.Count < udNum.Max
end;

procedure TOptForm.GetData(VAR pieXtra : TChartExtra);
// The Customize form directly modifies the pie chart, so most of
// the data doesn't need to be sent back to the main form. There
// are a handful of items stored in the "extra" structure that
// must be copied back by this routine
begin
  pieXtra.dCols.Assign(lbSlices.Items);
  pieXtra.dImage := fImgFile;
  pieXtra.dSlices := udNum.Position;
  IF rbImage.Checked THEN     pieXtra.dBak := 2
  ELSE IF rbGrad.Checked THEN pieXtra.dBak := 1
  ELSE                        pieXtra.dBak := 0;
end;

procedure TOptForm.ColorImage(Idx: Integer; Col: TColor);
VAR
  bBmp : TBitmap;
  cMsk : TColor;
begin
  bBmp := TBitmap.Create;
  try
    ilCols.GetBitmap(Idx, bBmp);
    IF Col = clFuchsia THEN
      cMsk := clAqua
    ELSE cMsk := clFuchsia;
    WITH bBmp DO
      begin
        Canvas.Brush.Color := cMsk;
        Canvas.FillRect(Rect(0, 0, Width, Height));
        Canvas.Brush.Color := Col;
        Canvas.Pen.Color   := clBlack;
        Canvas.Rectangle(1, 1, Pred(Width), Pred(Height));
      end;
    ilCols.ReplaceMasked(Idx, bBmp, cMsk);
  finally
    bBmp.Free;
  end;

end;

procedure TOptForm.FormActivate(Sender: TObject);
// If the two-button toolbar is narrower than the combobox below,
// adjust the spacing in the middle to right-align the right-hand
// button. If it's wider, widen the combobox and adjust sizes.
VAR
  Delta : Integer;
  L, R : Integer;
  Work : TRect;

  function Least : Integer;
  VAR dL, dR : Integer;
  begin
    dL := Self.Width - L;
    dR := Self.Width - R;
    IF dL < dR THEN
      Result := 0
    ELSE Result := 1;
  end;

begin
  Show;
  tbarSolid.Top := rbSolid.Top;
  tbarGrad.Top  := rbGrad.Top;
  tbarSolid.Left := cbGrad.Left;
  tbarGrad.Left := cbGrad.Left;
  tbarGrad.AutoSize := True;
  Delta := tbarGrad.Width - cbGrad.Width;
  IF Delta > 0 THEN
    begin
      Self.Width := Self.Width + Delta;
      cbGrad.Width := cbGrad.Width + Delta;
      cbImage.Width := cbImage.Width + Delta;
    end
  ELSE tbSpace.Width := tbSpace.Width - Delta;
  Bevel1.Width := tsBack.Width;
  Bevel2.Width := tsBack.Width;
  SystemParametersInfo(SPI_GETWORKAREA, 0, @Work, 0);
  Top := TForm(Owner).Top;
  L := TForm(Owner).Left;
  R := (Work.Right-Work.Left) -
    (TForm(Owner).Left + TForm(Owner).Width);
  IF Self.Width < L THEN
    Left := L-Width
  ELSE IF Self.Width < R THEN
    Left := TForm(Owner).Left+TForm(Owner).Width
  ELSE IF L > R THEN
    Left := Work.Left
  ELSE Left := Work.Right - Self.Width;
  WITH TIniFile.Create(ininame) DO
  try
    pcOpt.ActivePageIndex := ReadInteger(Self.Name + ' Settings',
      'Page', 0);
  finally
    Free;
  end;


end;

end.
