unit detailu;

// Copyright  2001 by Ziff Davis Media, Inc.
// Written by Neil J. Rubenking

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ComCtrls, ExtCtrls, dpShare;

type
  TDetailForm = class(TForm)
    RE          : TRichEdit;
    pnlButtons  : TPanel;
      btnOK     : TButton;
      btnPrint  : TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure btnPrintClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure SetData(shoTyp : Integer; Items, Colors : TStringList;
      slices : Integer; const fname : String); overload;
    procedure SetData(shoTyp : Integer; pieXtra : TChartExtra);
      overload;
  end;

var
  DetailForm: TDetailForm;

implementation

uses dpobj, allfuncs, formposu, mainu;

{$R *.dfm}

{ TDetailForm }

procedure TDetailForm.FormCreate(Sender: TObject);
begin
  GetPosFmIni(Ininame, Self, True);
end;

procedure TDetailForm.FormDestroy(Sender: TObject);
begin
  SetPosToIni(Ininame, Self, True);
end;

procedure TDetailForm.FormResize(Sender: TObject);
// The RichEdit control doesn't always refresh properly when the
// form resizes, so we do it explicitly
begin
  RE.Refresh;
end;

procedure TDetailForm.btnPrintClick(Sender: TObject);
begin
  IF MainForm.pd1.Execute THEN
    RE.Print(Application.Mainform.Caption);
end;

procedure TDetailForm.SetData(shoTyp : Integer; Items,
  Colors: TStringList; slices : Integer; const fname : String);
VAR
  N      : Integer;
  Sum    : Int64;
  S      : String;
  Num    : Integer;
  I      : Int64;
  Stream : TStringStream;

  function Slasher(const S : String) : String;
  // Double the slashes in the string, because RTF requires it
  begin
    Result := StringReplace(S, '\', '\\', [rfReplaceAll]);
  end;

  function BuildColorTable : String;
  VAR
    N  : Integer;
    TC : TColor;
  begin
    // Begin a color table
    Result := '{\colortbl ;'#13;
    // Add the red\green\blue values for each color
    FOR N := 0 TO Colors.Count-1 DO
      begin
        TC := TColor(Colors.Objects[N]);
        Result := Result + Format('\red%d\green%d\blue%d;'#13,
          [GetRValue(TC), GetGValue(TC), GetBValue(TC)]);
      end;
    // End the color table
    Result := Result + '}'#13;
  end;

begin
  RE.Lines.Clear;
  CASE shoTyp OF
    0 : Self.Caption := 'Detail for folder pie';
    1 : Self.Caption := 'Detail for file type pie';
    2 : Self.Caption := 'Largest files';
  END;
  IF Items.Count > slices THEN
    begin
      I := 0;
      FOR N := slices-1 TO Items.Count-1 DO
        Inc(I, TUObj(Items.Objects[N]).Value);
      Num := slices;
    end
  ELSE
    begin
      I := -1;
      Num := Items.Count;
    end;
  // Num is the number of items to process. If I is >= 0, I is the
  // total size of the items after Num. If not, there were no
  // items after Num.
  Sum := 0;
  FOR N := 0 TO Items.Count-1 DO
    Sum := Sum + TUObj(Items.Objects[N]).Value;
  // Sum is the total size of items in the list
  // Now create the RTF file in the string S.
  // Start with a minimial RTF header
  S := '{\rtf1\ansi'#13;
  // Add a font table
  S := S + '{\fonttbl{\f0\fnil\fprq2\fcharset2 Wingdings;}'+
    '{\f1\fswiss\fcharset0 Arial;}}'#13;
  // If this is not a "Largest files" report, build the color table
  IF ShoTyp < 2 THEN
    S := S + BuildColorTable;
  CASE shoTyp OF
    0 : begin // Folder pie report
      // Add tab stop information
      S := S+'\tx720\tx1440\tx3240\cf0\f1'#13;
      // Add the document header (the folder name)
      S := S+Format('\fs48\b\qc %s\b0\par'#13, [slasher(fName)]);
      // Add the subhead
      S := S+'\ql\fs32 Folder pie \par\fs24'#13;
      // Add the individual lines
      FOR N := 0 TO num-2 DO
        S := S + Format('\b[\cf%d\f0\fs28 n\cf0\f1\fs24]\b0'+
          '\tab %.02d%%\tab %s\tab %s\par'#13,
            // The colored wingdings rectangle...
            [Succ(N MOD Colors.Count),
            // ... the size percentage...
            Round((TUObj(Items.Objects[N]).Value*100)/Sum),
            // ... the actual size...
            StringForBytes(TUObj(Items.Objects[N]).Value),
            // ... and the subfolder name
            Slasher(Items[N])]);
      N := num-1;
      // Now add the last item. It may simply BE the last item, or it
      // may be an "Other" line
      IF N > 0 THEN
        begin
          IF I = -1 THEN
            S := S + Format('\b[\cf%d\f0\fs28 n\cf0\f1\fs24]\b0'+
              '\tab %.02d%%\tab %s\tab %s\par'#13,
              [Succ(N MOD Colors.Count),
              Round((TUObj(Items.Objects[N]).Value*100)/Sum),
              StringForBytes(TUObj(Items.Objects[N]).Value),
              Slasher(Items[N])])
          ELSE
            S := S + Format('\b[\cf%d\f0\fs28 n\cf0\f1\fs24]\b0'+
              '\tab %.02d%%\tab %s\tab %s\par'#13,
              [Succ(N MOD Colors.Count), Round((I*100)/Sum),
              StringForBytes(I), '"Other"']);
        end;
    end;
    1 : begin // File type pie report
      // Add tab stop information
      S := S+'\tx720\tx1440\tx3240\cf0\f1'#13;
      // Add the document header (the folder name)
      S := S+Format('\fs48\b\qc %s\b0\par'#13, [slasher(fName)]);
      // Add the subhead
      S := S+'\ql\fs32 File type pie \par\fs24'#13;
      // Add the individual lines
      FOR N := 0 TO num-2 DO
        S := S + Format('\b[\cf%d\f0\fs28 n\cf0\f1\fs24]\b0'+
          '\tab %.02d%%\tab %s\tab %s\par'#13,
          // The colored wingdings rectangle...
          [Succ(N MOD Colors.Count),
          // ... the size percentage...
          Round((TUObj(Items.Objects[N]).Value*100)/Sum),
          // ... the actual size...
          StringForBytes(TUObj(Items.Objects[N]).Value),
          // ... and the filetype name
          TDPTypeObj.AssnName(Items[N])]);
      N := num-1;
      // Now add the last item. It may simply BE the last item, or it
      // may be an "Other" line
      IF N > 0 THEN
        begin
          IF I = -1 THEN
            S := S + Format('\b[\cf%d\f0\fs28 n\cf0\f1\fs24]\b0'+
              '\tab %.02d%%\tab %s\tab %s\par'#13,
              [Succ(N MOD Colors.Count),
              Round((TUObj(Items.Objects[N]).Value*100)/Sum),
              StringForBytes(TUObj(Items.Objects[N]).Value),
              TDPTypeObj.AssnName(Items[N])])
          ELSE
            S := S + Format('\b[\cf%d\f0\fs28 n\cf0\f1\fs24]\b0'+
              '\tab %.02d%%\tab %s\tab %s\par'#13,
              [Succ(N MOD Colors.Count), Round((I*100)/Sum),
              StringForBytes(I), '"Other"']);
        end;
    end;
    2 : begin // Largest files report
      // Add tab stop information
      S := S+'\tx1440\cf0\f1'#13;
      // Add the document header (the folder name)
      S := S+Format('\fs48\b\qc %s\b0\par'#13, [slasher(fName)]);
      // Add the subhead
      S := S+'\ql\fs32 Largest files \par\fs24'#13;
      // Add the individual lines
      FOR N := 0 TO Items.Count-1 DO
        S := S + Format('%s\tab %s\par'#13,
          [StringForBytes(TUObj(Items.Objects[N]).Value),
            Slasher(Items[N])]);
    end;
  END;
  // To get the RTF data into the RichEdit, create a Tstringstream
  // and load the RichEdit from it.
  S := S + '}';
  Stream := TStringStream.Create(S);
  try
    RE.Lines.LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
  S := '';
end;

procedure TDetailForm.SetData(shoTyp: Integer; pieXtra: TChartExtra);
begin
  SetData(shoTyp, pieXtra.dList, pieXtra.dCols, pieXtra.dSlices,
    pieXtra.dTitle);
end;

end.
